@@ -47,9 +47,20 @@ class CLRuntimeError(RuntimeError):
 
 class CL(object):
     """Base OpenCL class.
+
+    Attributes:
+        _lib: handle to cffi.FFI object.
+        _handle: cffi handle to OpenCL object.
     """
     def __init__(self):
-        self.lib_ = cl.lib  # to hold the reference
+        self._lib = cl.lib  # to hold the reference
+        self._handle = None
+
+    @property
+    def handle(self):
+        """Returns cffi handle to OpenCL object.
+        """
+        return self._handle
 
     @staticmethod
     def extract_ptr_and_size(host_array, size):
@@ -76,7 +87,7 @@ class CL(object):
             n_events = len(wait_for)
             wait_list = cl.ffi.new("cl_event[]", n_events)
             for i, ev in enumerate(wait_for):
-                wait_list[i] = ev.event_
+                wait_list[i] = ev.handle
         else:
             n_events = 0
             wait_list = cl.NULL
@@ -87,7 +98,6 @@ class Event(CL):
     """Holds OpenCL event.
 
     Attributes:
-        event_: cffi OpenCL event handle.
         profiling_values:
             dictionary of profiling values
             if get_profiling_info was ever called;
@@ -101,9 +111,9 @@ class Event(CL):
         profiling_errors: dictionary of profiling errors
                           if get_profiling_info was ever called.
     """
-    def __init__(self, event_):
+    def __init__(self, handle):
         super(Event, self).__init__()
-        self.event_ = event_
+        self._handle = handle
 
     @staticmethod
     def wait_multi(wait_for, lib=cl.lib):
@@ -118,7 +128,7 @@ class Event(CL):
     def wait(self):
         """Waits on this event.
         """
-        Event.wait_multi((self,), self.lib_)
+        Event.wait_multi((self,), self._lib)
 
     def get_profiling_info(self, raise_exception=True):
         """Get profiling info of the event.
@@ -143,8 +153,8 @@ class Event(CL):
                      cl.CL_PROFILING_COMMAND_START,
                      cl.CL_PROFILING_COMMAND_END):
             vle[0] = 0
-            n = self.lib_.clGetEventProfilingInfo(
-                self.event_, name, sz, vle, cl.NULL)
+            n = self._lib.clGetEventProfilingInfo(
+                self.handle, name, sz, vle, cl.NULL)
             vles[name] = 1.0e-9 * vle[0] if not n else 0.0
             errs[name] = n
         self.profiling_values = vles
@@ -158,9 +168,9 @@ class Event(CL):
         return (vles, errs)
 
     def release(self):
-        if self.event_ is not None:
-            self.lib_.clReleaseEvent(self.event_)
-            self.event_ = None
+        if self.handle is not None:
+            self._lib.clReleaseEvent(self.handle)
+            self._handle = None
 
     def __del__(self):
         self.release()
@@ -170,23 +180,35 @@ class Queue(CL):
     """Holds OpenCL command queue.
 
     Attributes:
-        queue_: cffi OpenCL queue handle.
-        context: context assosiated with this queue.
-        device: device assosiated with this queue.
+        context: context associated with this queue.
+        device: device associated with this queue.
     """
     def __init__(self, context, device, flags):
         super(Queue, self).__init__()
-        self.queue_ = None
-        self.context = context
-        self.device = device
+        self._context = context
+        self._device = device
         err = cl.ffi.new("cl_int[]", 1)
-        self.queue_ = self.lib_.clCreateCommandQueue(
-            context.context_, device.device_, flags, err)
+        self._handle = self._lib.clCreateCommandQueue(
+            context.handle, device.handle, flags, err)
         if err[0]:
-            self.queue_ = None
+            self._handle = None
             raise CLRuntimeError("clCreateCommandQueue() failed with "
                                  "error %d" % (err[0]), err[0])
 
+    @property
+    def context(self):
+        """
+        context associated with this queue.
+        """
+        return self._context
+
+    @property
+    def device(self):
+        """
+        device associated with this queue.
+        """
+        return self._device
+
     def execute_kernel(self, kernel, global_size, local_size,
                        global_offset=None, wait_for=None, need_event=True):
         """Executes OpenCL kernel (calls clEnqueueNDRangeKernel).
@@ -226,8 +248,8 @@ class Queue(CL):
             global_work_offset = cl.ffi.new("size_t[]", n_dims)
             for i, sz in enumerate(global_offset):
                 global_work_offset[i] = sz
-        n = self.lib_.clEnqueueNDRangeKernel(
-            self.queue_, kernel.kernel_, n_dims, global_work_offset,
+        n = self._lib.clEnqueueNDRangeKernel(
+            self.handle, kernel.handle, n_dims, global_work_offset,
             global_work_size, local_work_size, n_events, wait_list, event)
         if n:
             raise CLRuntimeError("clEnqueueNDRangeKernel() failed with "
@@ -254,8 +276,8 @@ class Queue(CL):
         err = cl.ffi.new("cl_int[]", 1)
         event = cl.ffi.new("cl_event[]", 1) if need_event else cl.NULL
         wait_list, n_events = CL.get_wait_list(wait_for)
-        ptr = self.lib_.clEnqueueMapBuffer(
-            self.queue_, buf.buffer_, blocking, flags, offset, size,
+        ptr = self._lib.clEnqueueMapBuffer(
+            self.handle, buf.handle, blocking, flags, offset, size,
             n_events, wait_list, event, err)
         if err[0]:
             raise CLRuntimeError("clEnqueueMapBuffer() failed with "
@@ -277,8 +299,8 @@ class Queue(CL):
         """
         event = cl.ffi.new("cl_event[]", 1) if need_event else cl.NULL
         wait_list, n_events = CL.get_wait_list(wait_for)
-        n = self.lib_.clEnqueueUnmapMemObject(
-            self.queue_, buf.buffer_, cl.ffi.cast("void*", ptr),
+        n = self._lib.clEnqueueUnmapMemObject(
+            self.handle, buf.handle, cl.ffi.cast("void*", ptr),
             n_events, wait_list, event)
         if n:
             raise CLRuntimeError("clEnqueueUnmapMemObject() failed with "
@@ -304,8 +326,8 @@ class Queue(CL):
         event = cl.ffi.new("cl_event[]", 1) if need_event else cl.NULL
         wait_list, n_events = CL.get_wait_list(wait_for)
         host_ptr, size = CL.extract_ptr_and_size(host_array, size)
-        n = self.lib_.clEnqueueReadBuffer(
-            self.queue_, buf.buffer_, blocking, offset, size, host_ptr,
+        n = self._lib.clEnqueueReadBuffer(
+            self.handle, buf.handle, blocking, offset, size, host_ptr,
             n_events, wait_list, event)
         if n:
             raise CLRuntimeError("clEnqueueReadBuffer() failed with "
@@ -331,8 +353,8 @@ class Queue(CL):
         event = cl.ffi.new("cl_event[]", 1) if need_event else cl.NULL
         wait_list, n_events = CL.get_wait_list(wait_for)
         host_ptr, size = CL.extract_ptr_and_size(host_array, size)
-        n = self.lib_.clEnqueueWriteBuffer(
-            self.queue_, buf.buffer_, blocking, offset, size, host_ptr,
+        n = self._lib.clEnqueueWriteBuffer(
+            self.handle, buf.handle, blocking, offset, size, host_ptr,
             n_events, wait_list, event)
         if n:
             raise CLRuntimeError("clEnqueueReadBuffer() failed with "
@@ -342,21 +364,21 @@ class Queue(CL):
     def flush(self):
         """Flushes the queue.
         """
-        n = self.lib_.clFlush(self.queue_)
+        n = self._lib.clFlush(self.handle)
         if n:
             raise CLRuntimeError("clFlush() failed with error %d" % (n), n)
 
     def finish(self):
         """Waits for all previous commands issued to this queue to end.
         """
-        n = self.lib_.clFinish(self.queue_)
+        n = self._lib.clFinish(self.handle)
         if n:
             raise CLRuntimeError("clFinish() failed with error %d" % (n), n)
 
     def release(self):
-        if self.queue_ is not None:
-            self.lib_.clReleaseCommandQueue(self.queue_)
-            self.queue_ = None
+        if self.handle is not None:
+            self._lib.clReleaseCommandQueue(self.handle)
+            self._handle = None
 
     def __del__(self):
         self.release()
@@ -366,8 +388,7 @@ class Buffer(CL):
     """Holds OpenCL buffer.
 
     Attributes:
-        buffer_: cffi OpenCL buffer handle.
-        context: Context object assosiated with this buffer.
+        context: Context object associated with this buffer.
         flags: flags supplied for the creation of this buffer.
         host_array: host array reference, such as numpy array,
                     will be stored only if flags include CL_MEM_USE_HOST_PTR.
@@ -375,24 +396,52 @@ class Buffer(CL):
     """
     def __init__(self, context, flags, host_array, size=None):
         super(Buffer, self).__init__()
-        self.buffer_ = None
-        self.context = context
-        self.flags = flags
-        self.host_array = (host_array if flags & cl.CL_MEM_USE_HOST_PTR != 0
-                           else None)
+        self._context = context
+        self._flags = flags
+        self._host_array = (host_array if flags & cl.CL_MEM_USE_HOST_PTR != 0
+                            else None)
         host_ptr, size = CL.extract_ptr_and_size(host_array, size)
         err = cl.ffi.new("cl_int[]", 1)
-        self.buffer_ = self.lib_.clCreateBuffer(
-            context.context_, flags, size, host_ptr, err)
+        self._handle = self._lib.clCreateBuffer(
+            context.handle, flags, size, host_ptr, err)
         if err[0]:
-            self.buffer_ = None
+            self._handle = None
             raise CLRuntimeError("clCreateBuffer() failed with "
                                  "error %d" % (err[0]), err[0])
 
+    @property
+    def context(self):
+        """
+        Context object associated with this buffer.
+        """
+        return self._context
+
+    @property
+    def flags(self):
+        """
+        Flags supplied for the creation of this buffer.
+        """
+        return self._flags
+
+    @property
+    def host_array(self):
+        """
+        Host array reference, such as numpy array,
+        will be stored only if flags include CL_MEM_USE_HOST_PTR.
+        """
+        return self._host_array
+
+    @property
+    def size(self):
+        """
+        Size of the host array.
+        """
+        return self._size
+
     def release(self):
-        if self.buffer_ is not None:
-            self.lib_.clReleaseMemObject(self.buffer_)
-            self.buffer_ = None
+        if self.handle is not None:
+            self._lib.clReleaseMemObject(self.handle)
+            self._handle = None
 
     def __del__(self):
         self.release()
@@ -402,23 +451,35 @@ class Kernel(CL):
     """Holds OpenCL kernel.
 
     Attributes:
-        kernel_: cffi OpenCL kernel handle.
-        program: Program object assosiated with this kernel.
+        program: Program object associated with this kernel.
         name: kernel name in the program.
     """
     def __init__(self, program, name):
         super(Kernel, self).__init__()
-        self.kernel_ = None
-        self.program = program
-        self.name = name
+        self._program = program
+        self._name = name
         err = cl.ffi.new("cl_int[]", 1)
         ss = cl.ffi.new("char[]", name.encode("utf-8"))
-        self.kernel_ = self.lib_.clCreateKernel(program.program_, ss, err)
+        self._handle = self._lib.clCreateKernel(program.handle, ss, err)
         if err[0]:
-            self.kernel_ = None
+            self._handle = None
             raise CLRuntimeError("clCreateKernel() failed with "
                                  "error %d" % (err[0]), err[0])
 
+    @property
+    def program(self):
+        """
+        Program object associated with this kernel.
+        """
+        return self._program
+
+    @property
+    def name(self):
+        """
+        kernel name in the program.
+        """
+        return self._name
+
     def set_arg(self, idx, vle, size=None):
         """Sets kernel argument.
 
@@ -434,7 +495,7 @@ class Kernel(CL):
         """
         if isinstance(vle, Buffer):
             arg_value = cl.ffi.new("cl_mem[]", 1)
-            arg_value[0] = vle.buffer_
+            arg_value[0] = vle.handle
             arg_size = cl.ffi.sizeof("cl_mem")
         elif hasattr(vle, "__array_interface__"):
             arg_value = cl.ffi.cast("const void*",
@@ -452,15 +513,15 @@ class Kernel(CL):
             raise ValueError("vle should be of type Buffer, "
                              "numpy array, cffi pointer or None "
                              "in Kernel::set_arg()")
-        n = self.lib_.clSetKernelArg(self.kernel_, idx, arg_size, arg_value)
+        n = self._lib.clSetKernelArg(self.handle, idx, arg_size, arg_value)
         if n:
             raise CLRuntimeError("clSetKernelArg() failed with "
                                  "error %d" % (n), n)
 
     def release(self):
-        if self.kernel_ is not None:
-            self.lib_.clReleaseKernel(self.kernel_)
-            self.kernel_ = None
+        if self.handle is not None:
+            self._lib.clReleaseKernel(self.handle)
+            self._handle = None
 
     def __del__(self):
         self.release()
@@ -470,9 +531,8 @@ class Program(CL):
     """Holds OpenCL program.
 
     Attributes:
-        program_: cffi OpenCL program handle.
-        context: Context object assosiated with this program.
-        devices: list of Device objects assosiated with this program.
+        context: Context object associated with this program.
+        devices: list of Device objects associated with this program.
         build_logs: list of program build logs (same length as devices list).
         src: program source.
         include_dirs: list of include dirs.
@@ -480,21 +540,20 @@ class Program(CL):
     """
     def __init__(self, context, devices, src, include_dirs=(), options=""):
         super(Program, self).__init__()
-        self.program_ = None
-        self.context = context
-        self.devices = devices
-        self.src = src
-        self.include_dirs = list(include_dirs)
-        self.options = options
-        self.build_logs = []
+        self._context = context
+        self._devices = devices
+        self._src = src
+        self._include_dirs = list(include_dirs)
+        self._options = options
+        self._build_logs = []
         err = cl.ffi.new("cl_int[]", 1)
         ss = cl.ffi.new("char[]", src.encode("utf-8"))
         strings = cl.ffi.new("char*[]", 1)
         strings[0] = cl.ffi.cast("char*", ss)
-        self.program_ = self.lib_.clCreateProgramWithSource(
-            context.context_, 1, strings, cl.NULL, err)
+        self._handle = self._lib.clCreateProgramWithSource(
+            context.handle, 1, strings, cl.NULL, err)
         if err[0]:
-            self.program_ = None
+            self._handle = None
             raise CLRuntimeError("clCreateProgramWithSource() failed with "
                                  "error %d" % (err[0]), err[0])
         for dirnme in include_dirs:
@@ -504,6 +563,48 @@ class Program(CL):
                                  else "\'%s\'" % (dirnme))
         self._build_program(devices, options.strip().encode("utf-8"))
 
+    @property
+    def context(self):
+        """
+        Context object associated with this program.
+        """
+        return self._context
+
+    @property
+    def devices(self):
+        """
+        list of Device objects associated with this program.
+        """
+        return self._devices
+
+    @property
+    def build_logs(self):
+        """
+        list of program build logs (same length as devices list).
+        """
+        return self._build_logs
+
+    @property
+    def src(self):
+        """
+        program source.
+        """
+        return self._src
+
+    @property
+    def include_dirs(self):
+        """
+        list of include dirs.
+        """
+        return self._include_dirs
+
+    @property
+    def options(self):
+        """
+        additional build options.
+        """
+        return self._options
+
     def get_kernel(self, name):
         """Returns Kernel object from its name.
         """
@@ -513,15 +614,15 @@ class Program(CL):
         n_devices = len(devices)
         device_list = cl.ffi.new("cl_device_id[]", n_devices)
         for i, dev in enumerate(devices):
-            device_list[i] = dev.device_
-        n = self.lib_.clBuildProgram(self.program_, n_devices, device_list,
+            device_list[i] = dev.handle
+        n = self._lib.clBuildProgram(self.handle, n_devices, device_list,
                                      options, cl.NULL, cl.NULL)
         del self.build_logs[:]
         log = cl.ffi.new("char[]", 65536)
         sz = cl.ffi.new("size_t[]", 1)
         for dev in device_list:
-            m = self.lib_.clGetProgramBuildInfo(
-                self.program_, dev, cl.CL_PROGRAM_BUILD_LOG, 65536, log, sz)
+            m = self._lib.clGetProgramBuildInfo(
+                self.handle, dev, cl.CL_PROGRAM_BUILD_LOG, 65536, log, sz)
             if m or sz[0] <= 0:
                 self.build_logs.append("")
                 continue
@@ -534,9 +635,9 @@ class Program(CL):
                 (n, "\n".join(self.build_logs), self.src), n)
 
     def release(self):
-        if self.program_ is not None:
-            self.lib_.clReleaseProgram(self.program_)
-            self.program_ = None
+        if self.handle is not None:
+            self._lib.clReleaseProgram(self.handle)
+            self._handle = None
 
     def __del__(self):
         self.release()
@@ -546,31 +647,43 @@ class Context(CL):
     """Holds OpenCL context.
 
     Attributes:
-        context_: cffi OpenCL context handle.
-        platform: Platform object assosiated with this context.
-        devices: list of Device object assosiated with this context.
+        platform: Platform object associated with this context.
+        devices: list of Device object associated with this context.
     """
     def __init__(self, platform, devices):
         super(Context, self).__init__()
-        self.context_ = None
-        self.platform = platform
-        self.devices = devices
+        self._platform = platform
+        self._devices = devices
         props = cl.ffi.new("cl_context_properties[]", 3)
         props[0] = cl.CL_CONTEXT_PLATFORM
-        props[1] = cl.ffi.cast("cl_context_properties", platform.platform_)
+        props[1] = cl.ffi.cast("cl_context_properties", platform.handle)
         props[2] = 0
         err = cl.ffi.new("cl_int[]", 1)
         n_devices = len(devices)
         device_list = cl.ffi.new("cl_device_id[]", n_devices)
         for i, dev in enumerate(devices):
-            device_list[i] = dev.device_
-        self.context_ = self.lib_.clCreateContext(
+            device_list[i] = dev.handle
+        self._handle = self._lib.clCreateContext(
             props, n_devices, device_list, cl.NULL, cl.NULL, err)
         if err[0]:
-            self.context_ = None
+            self._handle = None
             raise CLRuntimeError("clCreateContext() failed with "
                                  "error %d" % (err[0]), err[0])
 
+    @property
+    def platform(self):
+        """
+        Platform object associated with this context.
+        """
+        return self._platform
+
+    @property
+    def devices(self):
+        """
+        List of Device object associated with this context.
+        """
+        return self._devices
+
     def create_queue(self, device, flags=0):
         """Creates Queue object for the supplied device.
 
@@ -598,7 +711,7 @@ class Context(CL):
 
     def create_program(self, src, include_dirs=(), options="", devices=None):
         """Creates and builds OpenCL program from source
-           for the supplied devices assosiated with this context.
+           for the supplied devices associated with this context.
 
         Parameters:
             src: program source.
@@ -613,9 +726,9 @@ class Context(CL):
                        src, include_dirs, options)
 
     def release(self):
-        if self.context_ is not None:
-            self.lib_.clReleaseContext(self.context_)
-            self.context_ = None
+        if self.handle is not None:
+            self._lib.clReleaseContext(self.handle)
+            self._handle = None
 
     def __del__(self):
         self.release()
@@ -625,10 +738,10 @@ class Device(CL):
     """OpenCL device.
 
     Attributes:
-        device_: cffi OpenCL device id.
-        platform: Platform object assosiated with this device.
+        platform: Platform object associated with this device.
         type: OpenCL type of the device (integer).
         name: OpenCL name of the device.
+        path: opencl4py device identifier,
         version: OpenCL version number of the device (float).
         version_string: OpenCL version string of the device.
         vendor: OpenCL vendor name of the device.
@@ -636,87 +749,185 @@ class Device(CL):
         memsize: global memory size of the device.
         memalign: align in bytes, required for clMapBuffer.
     """
-    def __init__(self, device_, platform):
+    def __init__(self, handle, platform, path):
         super(Device, self).__init__()
-        self.device_ = device_
-        self.platform = platform
+        self._handle = handle
+        self._platform = platform
+        self._path = path
 
         sz = cl.ffi.new("size_t[]", 1)
         tpe = cl.ffi.new("cl_device_type[]", 1)
-        n = self.lib_.clGetDeviceInfo(device_, cl.CL_DEVICE_TYPE,
+        n = self._lib.clGetDeviceInfo(handle, cl.CL_DEVICE_TYPE,
                                       8, tpe, sz)
-        self.type = tpe[0] if not n else None
+        self._type = tpe[0] if not n else None
 
         nme = cl.ffi.new("char[]", 256)
-        n = self.lib_.clGetDeviceInfo(device_, cl.CL_DEVICE_NAME,
+        n = self._lib.clGetDeviceInfo(handle, cl.CL_DEVICE_NAME,
                                       256, nme, sz)
-        self.name = ((b"".join(nme[0:sz[0] - 1])).decode("utf-8")
-                     if not n else None)
+        self._name = ((b"".join(nme[0:sz[0] - 1])).decode("utf-8")
+                      if not n else None)
 
-        n = self.lib_.clGetDeviceInfo(device_, cl.CL_DEVICE_OPENCL_C_VERSION,
+        n = self._lib.clGetDeviceInfo(handle, cl.CL_DEVICE_OPENCL_C_VERSION,
                                       256, nme, sz)
         s = (b"".join(nme[0:sz[0] - 1])).decode("utf-8") if not n else None
-        self.version_string = s
+        self._version_string = s
         n = len("OpenCL C ")
         m = s.find(" ", n)
         try:
-            self.version = float(s[n:m])
+            self._version = float(s[n:m])
         except ValueError:
-            self.version = 0.0
+            self._version = 0.0
 
-        n = self.lib_.clGetDeviceInfo(device_, cl.CL_DEVICE_VENDOR,
+        n = self._lib.clGetDeviceInfo(handle, cl.CL_DEVICE_VENDOR,
                                       256, nme, sz)
-        self.vendor = ((b"".join(nme[0:sz[0] - 1])).decode("utf-8")
-                       if not n else None)
+        self._vendor = ((b"".join(nme[0:sz[0] - 1])).decode("utf-8")
+                        if not n else None)
 
         vendor_id = cl.ffi.new("cl_uint[]", 1)
-        n = self.lib_.clGetDeviceInfo(device_, cl.CL_DEVICE_VENDOR_ID,
+        n = self._lib.clGetDeviceInfo(handle, cl.CL_DEVICE_VENDOR_ID,
                                       4, vendor_id, sz)
-        self.vendor_id = vendor_id[0] if not n else None
+        self._vendor_id = vendor_id[0] if not n else None
 
         memsize = cl.ffi.new("cl_ulong[]", 1)
-        n = self.lib_.clGetDeviceInfo(device_, cl.CL_DEVICE_GLOBAL_MEM_SIZE,
+        n = self._lib.clGetDeviceInfo(handle, cl.CL_DEVICE_GLOBAL_MEM_SIZE,
                                       8, memsize, sz)
-        self.memsize = memsize[0] if not n else None
+        self._memsize = memsize[0] if not n else None
 
         memalign = cl.ffi.new("cl_uint[]", 1)
-        n = self.lib_.clGetDeviceInfo(
-            device_, cl.CL_DEVICE_MEM_BASE_ADDR_ALIGN, 4, memalign, sz)
-        self.memalign = memalign[0] if not n else None
+        n = self._lib.clGetDeviceInfo(
+            handle, cl.CL_DEVICE_MEM_BASE_ADDR_ALIGN, 4, memalign, sz)
+        self._memalign = memalign[0] if not n else None
+
+    @property
+    def platform(self):
+        """
+        Platform object associated with this device.
+        """
+        return self._platform
+
+    @property
+    def type(self):
+        """
+        OpenCL type of the device (integer).
+        """
+        return self._type
+
+    @property
+    def name(self):
+        """
+        OpenCL name of the device.
+        """
+        return self._name
+
+    @property
+    def path(self):
+        """
+        opencl4py device identifier,
+        """
+        return self._path
+
+    @property
+    def version(self):
+        """
+        OpenCL version number of the device (float).
+        """
+        return self._version
+
+    @property
+    def version_string(self):
+        """
+        OpenCL version string of the device.
+        """
+        return self._version_string
+
+    @property
+    def vendor(self):
+        """
+        OpenCL vendor name of the device.
+        """
+        return self._vendor
+
+    @property
+    def vendor_id(self):
+        """
+        OpenCL vendor id of the device (integer).
+        """
+        return self._vendor_id
+
+    @property
+    def memsize(self):
+        """
+        global memory size of the device.
+        """
+        return self._memsize
+
+    @property
+    def memalign(self):
+        """
+        align in bytes, required for clMapBuffer.
+        """
+        return self._memalign
 
 
 class Platform(CL):
     """OpenCL platform.
 
     Attributes:
-        platform_: cffi OpenCL platform id.
         devices: list of Device objects available on this platform.
         name: OpenCL name of the platform.
+        path: opencl4py platform identifier.
     """
-    def __init__(self, platform_):
+    def __init__(self, handle, path):
         super(Platform, self).__init__()
-        self.platform_ = platform_
+        self._handle = handle
+        self._path = path
 
         sz = cl.ffi.new("size_t[]", 1)
         nme = cl.ffi.new("char[]", 256)
-        n = self.lib_.clGetPlatformInfo(platform_, cl.CL_PLATFORM_NAME,
+        n = self._lib.clGetPlatformInfo(handle, cl.CL_PLATFORM_NAME,
                                         256, nme, sz)
-        self.name = ((b"".join(nme[0:sz[0] - 1])).decode("utf-8")
-                     if not n else None)
+        self._name = ((b"".join(nme[0:sz[0] - 1])).decode("utf-8")
+                      if not n else None)
 
         nn = cl.ffi.new("cl_uint[]", 1)
-        n = self.lib_.clGetDeviceIDs(platform_, cl.CL_DEVICE_TYPE_ALL,
+        n = self._lib.clGetDeviceIDs(handle, cl.CL_DEVICE_TYPE_ALL,
                                      0, cl.NULL, nn)
         if n:
             raise CLRuntimeError("clGetDeviceIDs() failed "
                                  "with error %d" % (n), n)
         ids = cl.ffi.new("cl_device_id[]", nn[0])
-        n = self.lib_.clGetDeviceIDs(platform_, cl.CL_DEVICE_TYPE_ALL,
+        n = self._lib.clGetDeviceIDs(handle, cl.CL_DEVICE_TYPE_ALL,
                                      nn[0], ids, nn)
         if n:
             raise CLRuntimeError("clGetDeviceIDs() failed "
                                  "with error %d" % (n), n)
-        self.devices = list(Device(dev_id, self) for dev_id in ids)
+        self._devices = list(Device(dev_id, self,
+                                    "%s:%d" % (self.path, dev_num))
+                             for dev_id, dev_num in zip(ids, range(len(ids))))
+
+    @property
+    def devices(self):
+        """
+        List of Device objects available on this platform.
+        """
+        return self._devices
+
+    @property
+    def name(self):
+        """
+        OpenCL name of the platform.
+        """
+        return self._name
+
+    @property
+    def path(self):
+        """
+        opencl4py platform identifier.
+        """
+        return self._path
+
+    def __iter__(self):
+        return iter(self.devices)
 
     def create_context(self, devices):
         """Creates OpenCL context on this platform and selected devices.
@@ -740,16 +951,24 @@ class Platforms(CL):
         cl.initialize()
         super(Platforms, self).__init__()
         nn = cl.ffi.new("cl_uint[]", 1)
-        n = self.lib_.clGetPlatformIDs(0, cl.NULL, nn)
+        n = self._lib.clGetPlatformIDs(0, cl.NULL, nn)
         if n:
             raise CLRuntimeError("clGetPlatformIDs() failed "
                                  "with error %d" % (n), n)
         ids = cl.ffi.new("cl_platform_id[]", nn[0])
-        n = self.lib_.clGetPlatformIDs(nn[0], ids, nn)
+        n = self._lib.clGetPlatformIDs(nn[0], ids, nn)
         if n:
             raise CLRuntimeError("clGetPlatformIDs() failed "
                                  "with error %d" % (n), n)
-        self.platforms = list(Platform(p_id) for p_id in ids)
+        self._platforms = list(Platform(p_id, str(p_num))
+                               for p_id, p_num in zip(ids, range(len(ids))))
+
+    @property
+    def platforms(self):
+        return self._platforms
+
+    def __iter__(self):
+        return iter(self.platforms)
 
     def dump_devices(self):
         """Returns string with information about OpenCL platforms and devices.
